Struct string_interner::Sym[][src]

pub struct Sym(_);

Symbol type used by the DefaultStringInterner.

Note

This special symbol type has a memory footprint of 32 bits and allows for certain space optimizations such as using it within an option: Option<Sym>