Trait string_interner::Symbol [−][src]
pub trait Symbol: Copy + Ord + Eq { fn from_usize(val: usize) -> Self; fn to_usize(self) -> usize; }
Types implementing this trait are able to act as symbols for string interners.
Symbols are returned by StringInterner::get_or_intern and allow look-ups of the
original string contents with StringInterner::resolve.
Note
Optimal symbols allow for efficient comparisons and have a small memory footprint.
Required Methods
fn from_usize(val: usize) -> Self
fn to_usize(self) -> usize
Returns the usize representation of self.
Implementations on Foreign Types
impl Symbol for usize[src]
impl Symbol for usizeImplementors
impl Symbol for Sym