[][src]Struct string_interner::StringInterner

pub struct StringInterner<S = DefaultSymbol, B = DefaultBackend, H = DefaultHashBuilder> where
    S: Symbol,
    B: Backend<S>,
    H: BuildHasher
{ /* fields omitted */ }

Data structure to intern and resolve strings.

Caches strings efficiently, with minimal memory footprint and associates them with unique symbols. These symbols allow constant time comparisons and look-ups to the underlying interned strings.

The following API covers the main functionality:

Implementations

impl<S, B, H> StringInterner<S, B, H> where
    S: Symbol,
    B: Backend<S>,
    H: BuildHasher + Default
[src]

pub fn new() -> Self[src]

Creates a new empty StringInterner.

pub fn with_capacity(cap: usize) -> Self[src]

Creates a new StringInterner with the given initial capacity.

impl<S, B, H> StringInterner<S, B, H> where
    S: Symbol,
    B: Backend<S>,
    H: BuildHasher
[src]

pub fn with_hasher(hash_builder: H) -> Self[src]

Creates a new empty StringInterner with the given hasher.

pub fn with_capacity_and_hasher(cap: usize, hash_builder: H) -> Self[src]

Creates a new empty StringInterner with the given initial capacity and the given hasher.

pub fn len(&self) -> usize[src]

Returns the number of strings interned by the interner.

pub fn is_empty(&self) -> bool[src]

Returns true if the string interner has no interned strings.

pub fn get<T>(&self, string: T) -> Option<S> where
    T: AsRef<str>, 
[src]

Returns the symbol for the given string if any.

Can be used to query if a string has already been interned without interning.

pub fn get_or_intern<T>(&mut self, string: T) -> S where
    T: AsRef<str>, 
[src]

Interns the given string.

Returns a symbol for resolution into the original string.

pub fn resolve(&self, symbol: S) -> Option<&str>[src]

Returns the string for the given symbol if any.