Struct string_interner::StringInterner[][src]

pub struct StringInterner<S, H = RandomState> where
    S: Symbol,
    H: BuildHasher
{ /* fields omitted */ }

Caches strings efficiently, with minimal memory footprint and associates them with unique symbols. These symbols allow constant time comparisons and look-ups to the underlying interned strings.

Methods

impl<S> StringInterner<S> where
    S: Symbol
[src]

Creates a new empty StringInterner.

Creates a new StringInterner with the given initial capacity.

Returns the number of elements the StringInterner can hold without reallocating.

Reserves capacity for at least additional more elements to be interned into self.

The collection may reserve more space to avoid frequent allocations. After calling reserve, capacity will be greater than or equal to self.len() + additional. Does nothing if capacity is already sufficient.

impl<S, H> StringInterner<S, H> where
    S: Symbol,
    H: BuildHasher
[src]

Creates a new empty StringInterner with the given hasher.

Creates a new empty StringInterner with the given initial capacity and the given hasher.

Interns the given value.

Returns a symbol to access it within this interner.

This either copies the contents of the string (e.g. for str) or moves them into this interner (e.g. for String).

Returns the string slice associated with the given symbol if available, otherwise returns None.

Returns the string associated with the given symbol.

Note

This does not check whether the given symbol has an associated string for the given string interner instance.

Safety

This will result in undefined behaviour if the given symbol had no associated string for this interner instance.

Returns the symbol associated with the given string for this interner if existent, otherwise returns None.

Returns the number of uniquely interned strings within this interner.

Returns true if the string interner holds no elements.

Important traits for Iter<'a, S>

Returns an iterator over the interned strings.

Important traits for Values<'a, S>

Returns an iterator over all intern indices and their associated strings.

Shrinks the capacity of the interner as much as possible.